@@ -15,12 +15,71 @@
 #include "clang/Basic/Version.h"
 #include "clang/Basic/SourceManager.h"
 #include "clang/Sema/Sema.h"
+#if CLANG_VERSION_MAJOR > 18
+#include "clang/Sema/SemaOpenMP.h"
+#endif
 
 namespace clad_compat {
 
 using namespace clang;
 using namespace llvm;
 
+// clang-21
+#if CLANG_VERSION_MAJOR < 21
+#define CLAD_COMPAT_CLANG21_AtEndOfTUParam
+#define CLAD_COMPAT_CLANG21_CSSExtendKWLocExtraParam(V) (V),
+#define CLAD_COMPAT_CLANG21_StringLiteralParams(V) &(V)[0], (V).size()
+#define CLAD_COMPAT_CLANG21_StringLiteralParamsRange , SourceRange()
+#define CLAD_COMPAT_CLANG21_getTrailingRequiresClause(FD)                      \
+  const_cast<FunctionDecl*>(FD)->getTrailingRequiresClause()
+#define CLAD_COMPAT_CLANG21_getTrailingRequiresExpr(FD)                        \
+  (FD)->getTrailingRequiresClause()
+#define CLAD_COMPAT_CLANG21_UpdateTrailingRequiresClause(Trailing, V)          \
+  (Trailing) = (V)
+#define CLAD_COMPAT_CLANG21_TemplateKeywordParam , /*TemplateKeyword=*/false
+#else
+#define CLAD_COMPAT_CLANG21_AtEndOfTUParam , /*AtEndOfTU=*/true
+#define CLAD_COMPAT_CLANG21_CSSExtendKWLocExtraParam(V)
+#define CLAD_COMPAT_CLANG21_StringLiteralParams(V) (V)
+#define CLAD_COMPAT_CLANG21_StringLiteralParamsRange
+#define CLAD_COMPAT_CLANG21_getTrailingRequiresClause(FD)                      \
+  (FD)->getTrailingRequiresClause()
+#define CLAD_COMPAT_CLANG21_getTrailingRequiresExpr(FD)                        \
+  (FD)->getTrailingRequiresClause().ConstraintExpr
+#define CLAD_COMPAT_CLANG21_UpdateTrailingRequiresClause(Trailing, V)          \
+  (Trailing).ConstraintExpr = (V)
+#define CLAD_COMPAT_CLANG21_TemplateKeywordParam
+#endif
+
+// clang-21 OpenMPReductionClauseModifiers  got extra argument
+#if CLANG_VERSION_MAJOR < 21
+#define CLAD_COMPAT_CLANG21_getModifier(Clause) (Clause)->getModifier()
+#else
+#define CLAD_COMPAT_CLANG21_getModifier(Clause)                                \
+  {(Clause)->getModifier(), (Clause)->getOriginalSharingModifier()}
+#endif
+
+// clang-20 Clause varlist typo
+#if CLANG_VERSION_MAJOR < 20
+#define CLAD_COMPAT_CLANG20_getvarlist(Clause) (Clause)->varlists()
+#else
+#define CLAD_COMPAT_CLANG20_getvarlist(Clause) (Clause)->varlist()
+#endif
+
+// clang-20 clang::Sema::AA_Casting became scoped
+#if CLANG_VERSION_MAJOR < 20
+#define CLAD_COMPAT_CLANG20_SemaAACasting clang::Sema::AA_Casting
+#else
+#define CLAD_COMPAT_CLANG20_SemaAACasting clang::AssignmentAction::Casting
+#endif
+
+// clang-19 SemaOpenMP was introduced
+#if CLANG_VERSION_MAJOR < 19
+#define CLAD_COMPAT_CLANG19_SemaOpenMP(Sema) (Sema)
+#else
+#define CLAD_COMPAT_CLANG19_SemaOpenMP(Sema) ((Sema).OpenMP())
+#endif
+
 // clang-18 CXXThisExpr got extra argument
 
 #if CLANG_VERSION_MAJOR > 17
@@ -233,56 +292,13 @@ CUDAKernelCallExpr_Create(const ASTContext& Ctx, Expr* Fn, CallExpr* Config,
    #define CLAD_COMPAT_CLANG8_CallExpr_ExtraParams ,Node->getFPFeatures(),Node->getNumArgs(),Node->getADLCallKind()
 #endif
 
-// Clang 11 add one param to CXXOperatorCallExpr_Create, ADLCallKind, and many other constructor and Create changes
-
-#if CLANG_VERSION_MAJOR < 11
-
-static inline void ExprSetDeps(Expr* result, Expr* Node) {
-   result->setValueDependent(Node->isValueDependent());
-   result->setTypeDependent(Node->isTypeDependent());
-}
-
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParams /**/
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParamsPar /**/
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParamsUse /**/
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParamsOverride FPOptions
-   #define CLAD_COMPAT_CLANG11_LangOptions_EtraParams /**/
-   #define CLAD_COMPAT_CLANG11_Ctx_ExtraParams /**/
-   #define CLAD_COMPAT_CREATE11(CLASS, CTORARGS) (new (Ctx) CLASS CTORARGS)
-   #define CLAD_COMPAT_CLANG11_CompoundAssignOperator_EtraParams_Removed Node->getComputationLHSType(),Node->getComputationResultType(),
-   #define CLAD_COMPAT_CLANG11_CompoundAssignOperator_EtraParams_Moved /**/
-   #define CLAD_COMPAT_CLANG11_ChooseExpr_EtraParams_Removed ,Node->isTypeDependent(),Node->isValueDependent()
-   #define CLAD_COMPAT_CLANG11_WhileStmt_ExtraParams /**/
-
-#elif CLANG_VERSION_MAJOR >= 11
-
-struct ExprDependenceAccessor : public Expr {
-   void setDependence(ExprDependence Deps) {
-      Expr::setDependence(Deps);
-   }
-};
-
 static inline void ExprSetDeps(Expr* result, Expr* Node) {
+  struct ExprDependenceAccessor : public Expr {
+    void setDependence(ExprDependence Deps) { Expr::setDependence(Deps); }
+  };
    ((ExprDependenceAccessor*)result)->setDependence(Node->getDependence());
 }
 
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParams ,Node->getADLCallKind()
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParamsPar ,clang::CallExpr::ADLCallKind UsesADL
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParamsUse ,UsesADL
-   #define CLAD_COMPAT_CLANG11_CXXOperatorCallExpr_Create_ExtraParamsOverride FPOptionsOverride
-   #if CLANG_VERSION_MAJOR >= 16
-      #define CLAD_COMPAT_CLANG11_LangOptions_EtraParams /**/
-   #else
-      #define CLAD_COMPAT_CLANG11_LangOptions_EtraParams Ctx.getLangOpts()
-   #endif
-   #define CLAD_COMPAT_CLANG11_Ctx_ExtraParams Ctx,
-   #define CLAD_COMPAT_CREATE11(CLASS, CTORARGS) (CLASS::Create CTORARGS)
-   #define CLAD_COMPAT_CLANG11_CompoundAssignOperator_EtraParams_Removed /**/
-   #define CLAD_COMPAT_CLANG11_CompoundAssignOperator_EtraParams_Moved ,Node->getComputationLHSType(),Node->getComputationResultType()
-   #define CLAD_COMPAT_CLANG11_ChooseExpr_EtraParams_Removed /**/
-   #define CLAD_COMPAT_CLANG11_WhileStmt_ExtraParams ,Node->getLParenLoc(),Node->getRParenLoc()
-#endif
-
 // Compatibility helper function for creation CXXMemberCallExpr.
 // Clang 12 and above use two extra param.
 
@@ -323,7 +339,6 @@ static inline T GetResult(ActionResult<T> Res)
    return Res.get();
 }
 
-// Clang 10 change add new param in getConstantArrayType.
 // clang 18 clang::ArrayType::ArraySizeModifier became clang::ArraySizeModifier
 #if CLANG_VERSION_MAJOR < 18
 static inline QualType
@@ -331,12 +346,8 @@ getConstantArrayType(const ASTContext& Ctx, QualType EltTy,
                      const APInt& ArySize, const Expr* SizeExpr,
                      clang::ArrayType::ArraySizeModifier ASM,
                      unsigned IndexTypeQuals) {
-#if CLANG_VERSION_MAJOR < 10
-   return Ctx.getConstantArrayType(EltTy, ArySize, ASM, IndexTypeQuals);
-#elif CLANG_VERSION_MAJOR >= 10
   return Ctx.getConstantArrayType(EltTy, ArySize, SizeExpr, ASM,
                                   IndexTypeQuals);
-#endif
 }
 #else
 static inline QualType
@@ -348,39 +359,6 @@ getConstantArrayType(const ASTContext& Ctx, QualType EltTy,
 }
 #endif
 
-// Clang 10 add new last param TrailingRequiresClause in FunctionDecl::Create
-
-#if CLANG_VERSION_MAJOR < 10
-   #define CLAD_COMPAT_CLANG10_FunctionDecl_Create_ExtraParams(x) /**/
-#elif CLANG_VERSION_MAJOR >= 10
-#define CLAD_COMPAT_CLANG10_FunctionDecl_Create_ExtraParams(x)                 \
-  , ((x) ? VB.Clone((x)) : nullptr)
-#endif
-
-// Clang 10 remove GetTemporaryExpr(). Use getSubExpr() instead
-
-#if CLANG_VERSION_MAJOR < 10
-   #define CLAD_COMPAT_CLANG10_GetTemporaryExpr(x) (x)->GetTemporaryExpr()
-#elif CLANG_VERSION_MAJOR >= 10
-   #define CLAD_COMPAT_CLANG10_GetTemporaryExpr(x) (x)->getSubExpr()?Clone((x)->getSubExpr()):nullptr
-#endif
-
-// Clang 10 add one param to StmtExpr constructor: unsigned TemplateDepth
-
-#if CLANG_VERSION_MAJOR < 10
-   #define CLAD_COMPAT_CLANG10_StmtExpr_Create_ExtraParams /**/
-#elif CLANG_VERSION_MAJOR >= 10
-   #define CLAD_COMPAT_CLANG10_StmtExpr_Create_ExtraParams ,Node->getTemplateDepth()
-#endif
-
-// Clang 11 add one extra param in UnaryOperator constructor.
-
-#if CLANG_VERSION_MAJOR < 11
-   #define CLAD_COMPAT_CLANG11_UnaryOperator_ExtraParams /**/
-#elif CLANG_VERSION_MAJOR >= 11
-   #define CLAD_COMPAT_CLANG11_UnaryOperator_ExtraParams ,Node->getFPOptionsOverride()
-#endif
-
 // Clang 12 rename DeclaratorContext::LambdaExprContext to DeclaratorContext::LambdaExpr.
 // Clang 15 add one extra param to clang::Declarator() - const ParsedAttributesView & DeclarationAttrs
 
@@ -422,18 +400,6 @@ getConstantArrayType(const ASTContext& Ctx, QualType EltTy,
    #define CLAD_COMPAT_CLANG12_LR_ExtraParams(Node) ,Node->getLParenLoc(),Node->getRParenLoc()
 #endif
 
-/// clang >= 11 added more source locations parameters in `Sema::ActOnWhileStmt`
-static inline StmtResult
-Sema_ActOnWhileStmt(Sema& SemaRef, Sema::ConditionResult cond, Stmt* body) {
-  SourceLocation noLoc;
-#if CLANG_VERSION_MAJOR < 11
-  return SemaRef.ActOnWhileStmt(/*WhileLoc=*/noLoc, cond, body);
-#elif CLANG_VERSION_MAJOR >= 11
-  return SemaRef.ActOnWhileStmt(/*WhileLoc=*/noLoc, /*LParenLoc=*/noLoc, cond,
-                                /*RParenLoc=*/noLoc, body);
-#endif
-}
-
 /// Clang >= 12 has more source locations parameters in `Sema::ActOnStartOfSwitchStmt`
 static inline StmtResult
 Sema_ActOnStartOfSwitchStmt(Sema& SemaRef, Stmt* initStmt,
@@ -449,16 +415,6 @@ Sema_ActOnStartOfSwitchStmt(Sema& SemaRef, Stmt* initStmt,
 #endif
 }
 
-/// Clang 11 added an extra parameter for immediate invocation in
-/// ConstantExpr::Create
-#if CLANG_VERSION_MAJOR < 11
-#define CLAD_COMPAT_ConstantExpr_Create_ExtraParams\
-  , Node->getResultStorageKind()
-#else
-#define CLAD_COMPAT_ConstantExpr_Create_ExtraParams\
-  , Node->getResultStorageKind(), Node->isImmediateInvocation()
-#endif
-
 #if CLANG_VERSION_MAJOR < 13
 #define CLAD_COMPAT_ExprValueKind_R_or_PR_Value ExprValueKind::VK_RValue
 #elif CLANG_VERSION_MAJOR >= 13
@@ -517,16 +473,6 @@ static inline MemberExpr* BuildMemberExpr(
 }
 #endif
 
-#if CLANG_VERSION_MAJOR < 10
-static inline Expr* GetSubExpr(const MaterializeTemporaryExpr* MTE) {
-  return MTE->GetTemporaryExpr();
-}
-#else
-static inline Expr* GetSubExpr(const MaterializeTemporaryExpr* MTE) {
-  return MTE->getSubExpr();
-}
-#endif
-
 static inline QualType
 CXXMethodDecl_GetThisObjectType(Sema& semaRef, const CXXMethodDecl* MD) {
 // clang-18 renamed getThisObjectType to getFunctionObjectParameterType
@@ -594,19 +540,18 @@ ArraySize_GetValue(const std::optional<const Expr*>& opt) {
 }
 #endif
 
+#if CLANG_VERSION_MAJOR < 16
+#define CLAD_COMPAT_CLANG16_LangOptions_ExtraParams Ctx.getLangOpts()
+#else
+#define CLAD_COMPAT_CLANG16_LangOptions_ExtraParams /**/
+#endif
+
 #if CLANG_VERSION_MAJOR < 13
 static inline bool IsPRValue(const Expr* E) { return E->isRValue(); }
 #else
 static inline bool IsPRValue(const Expr* E) { return E->isPRValue(); }
 #endif
 
-#if CLANG_VERSION_MAJOR >= 9
-#define CLAD_COMPAT_CLANG9_CXXDefaultArgExpr_getUsedContext_Param(Node)               \
-  , Node->getUsedContext()
-#else
-#define CLAD_COMPAT_CLANG9_CXXDefaultArgExpr_getUsedContext_Param(Node) /**/
-#endif
-
 #if CLANG_VERSION_MAJOR >= 16
 #define CLAD_COMPAT_CLANG16_CXXDefaultArgExpr_getRewrittenExpr_Param(Node)     \
   , Node->getRewrittenExpr()
@@ -660,5 +605,12 @@ static inline const DeclSpec& Sema_ActOnStartOfLambdaDefinition_ScopeOrDeclSpec(
   ,Node->isTransparent()
 #endif
 
+// Clang 19 renamed the enum representing template resolution results
+#if CLANG_VERSION_MAJOR >= 19
+#define CLAD_COMPAT_TemplateSuccess TemplateDeductionResult::Success
+#else
+#define CLAD_COMPAT_TemplateSuccess Sema::TDK_Success
+#endif
+
 } // namespace clad_compat
 #endif //CLAD_COMPATIBILITY
