@@ -7,22 +7,37 @@
 #ifndef CLAD_VISITOR_BASE_H
 #define CLAD_VISITOR_BASE_H
 
-namespace clad {
-  class DerivativeBuilder;
-}
-
 #include "Compatibility.h"
 #include "DerivativeBuilder.h"
+#include "clad/Differentiator/CladUtils.h"
 
 #include "clang/AST/RecursiveASTVisitor.h"
 #include "clang/AST/StmtVisitor.h"
+#include "clang/AST/Type.h"
+#include "clang/Basic/OperatorKinds.h"
+#include "clang/Sema/DeclSpec.h"
+#include "clang/Sema/Ownership.h"
 #include "clang/Sema/ParsedAttr.h"
 #include "clang/Sema/Sema.h"
+
+#include "llvm/ADT/ArrayRef.h"
+#include "llvm/ADT/StringRef.h"
+#include "llvm/Support/PrettyStackTrace.h"
+
 #include <array>
 #include <stack>
 #include <unordered_map>
 
+namespace clang {
+class NestedNameSpecifier;
+} // namespace clang
+
+namespace llvm {
+class raw_ostream;
+} // namespace llvm
+
 namespace clad {
+  class MultiplexExternalRMVSource;
   /// A class that represents the result of Visit of ForwardModeVisitor.
   /// Stmt() allows to access the original (cloned) Stmt and Stmt_dx() allows
   /// to access its derivative (if exists, otherwise null). If Visit produces
@@ -30,15 +45,12 @@ namespace clad {
   class StmtDiff {
   private:
     std::array<clang::Stmt*, 2> data;
-    clang::Stmt* m_DerivativeForForwSweep;
     clang::Stmt* m_ValueForRevSweep;
 
   public:
     StmtDiff(clang::Stmt* orig = nullptr, clang::Stmt* diff = nullptr,
-             clang::Stmt* forwSweepDiff = nullptr,
              clang::Stmt* valueForRevSweep = nullptr)
-        : m_DerivativeForForwSweep(forwSweepDiff),
-          m_ValueForRevSweep(valueForRevSweep) {
+        : m_ValueForRevSweep(valueForRevSweep) {
       data[1] = orig;
       data[0] = diff;
     }
@@ -54,11 +66,10 @@ namespace clad {
 
     void updateStmt(clang::Stmt* S) { data[1] = S; }
     void updateStmtDx(clang::Stmt* S) { data[0] = S; }
+    void updateRevSweep(clang::Stmt* S) { m_ValueForRevSweep = S; }
     // Stmt_dx goes first!
     std::array<clang::Stmt*, 2>& getBothStmts() { return data; }
 
-    clang::Stmt* getForwSweepStmt_dx() { return m_DerivativeForForwSweep; }
-
     clang::Expr* getRevSweepAsExpr() {
       return llvm::cast_or_null<clang::Expr>(getRevSweepStmt());
     }
@@ -70,12 +81,6 @@ namespace clad {
         return data[1];
       return m_ValueForRevSweep;
     }
-
-    clang::Expr* getForwSweepExpr_dx() {
-      return llvm::cast_or_null<clang::Expr>(m_DerivativeForForwSweep);
-    }
-
-    void setForwSweepStmt_dx(clang::Stmt* S) { m_DerivativeForForwSweep = S; }
   };
 
   template <typename T> class DeclDiff {
@@ -130,10 +135,6 @@ namespace clad {
     /// A stack of all the blocks where the statements of the gradient function
     /// are stored (e.g., function body, if statement blocks).
     std::vector<Stmts> m_Blocks;
-    /// Stores output variables for vector-valued functions
-    VectorOutputs m_VectorOutput;
-    /// The functor type that is currently being differentiated, if any.
-    const clang::CXXRecordDecl* m_Functor = nullptr;
     /// Stores derivative expression of the implicit `this` pointer.
     ///
     /// In the forward mode, `this` pointer derivative expression is of pointer
@@ -143,6 +144,9 @@ namespace clad {
     // expression to be of object type in the reverse mode as well.
     clang::Expr* m_ThisExprDerivative = nullptr;
 
+    /// The currently visited statement. Useful for crash pretty-printing.
+    const clang::Stmt* m_CurVisitedStmt = nullptr;
+
     /// A function used to wrap result of visiting E in a lambda. Returns a call
     /// to the built lambda. Func is a functor that will be invoked inside
     /// lambda scope and block. Statements inside lambda are expected to be
@@ -201,7 +205,7 @@ namespace clad {
     }
     /// For a qualtype QT returns if it's type is Array or Pointer Type
     static bool isArrayOrPointerType(const clang::QualType QT) {
-      return QT->isArrayType() || QT->isPointerType();
+      return utils::isArrayOrPointerType(QT);
     }
 
     clang::CompoundStmt* MakeCompoundStmt(const Stmts& Stmts);
@@ -278,7 +282,31 @@ namespace clad {
     clang::Expr* BuildOp(clang::BinaryOperatorKind OpCode, clang::Expr* L,
                          clang::Expr* R, clang::SourceLocation OpLoc = noLoc);
 
+    /// A shorthand to simplify syntax for creation of CXXOperatorCallExpr.
+    /// We need it because Clang doesn't have a common ActOn- function to
+    /// generate operator calls based on the operator kind. \param[in] OOK The
+    /// kind of the operator. \param[in] ArgExprs The arguments of the operator.
+    /// \param[in] OpLoc The source location, if necessary.
+    /// \returns An expression of the built operator.
+    clang::Expr* BuildOperatorCall(clang::OverloadedOperatorKind OOK,
+                                   llvm::MutableArrayRef<clang::Expr*> ArgExprs,
+                                   clang::SourceLocation OpLoc = noLoc);
+    /// Function to resolve Unary Minus. If the leftmost operand
+    /// has a Unary Minus then adds parens before adding the unary minus.
+    /// \param[in] E Expression fed to the recursive call.
+    /// \param[in] OpLoc Location to add Unary Minus if needed.
+    /// \returns Expression with correct Unary Operator placement.
+    clang::Expr* ResolveUnaryMinus(clang::Expr* E, clang::SourceLocation OpLoc);
     clang::Expr* BuildParens(clang::Expr* E);
+    /// Sets Init as the initializer of the declaration VD and compute its
+    /// initialization kind.
+    ///\param[in] VD - variable declaration
+    ///\param[in] Init - can be nullptr, then only initialization kind is
+    /// computed.
+    ///\param[in] DirectInit - tells whether the initialization is
+    /// direct.
+    void SetDeclInit(clang::VarDecl* VD, clang::Expr* Init = nullptr,
+                     bool DirectInit = false);
     /// Builds variable declaration to be used inside the derivative
     /// body.
     /// \param[in] Type The type of variable declaration to build.
@@ -293,9 +321,7 @@ namespace clad {
     clang::VarDecl*
     BuildVarDecl(clang::QualType Type, clang::IdentifierInfo* Identifier,
                  clang::Scope* scope, clang::Expr* Init = nullptr,
-                 bool DirectInit = false, clang::TypeSourceInfo* TSI = nullptr,
-                 clang::VarDecl::InitializationStyle IS =
-                     clang::VarDecl::InitializationStyle::CInit);
+                 bool DirectInit = false, clang::TypeSourceInfo* TSI = nullptr);
     /// Builds variable declaration to be used inside the derivative
     /// body.
     /// \param[in] Type The type of variable declaration to build.
@@ -307,12 +333,11 @@ namespace clad {
     /// C style initalization.
     /// \param[in] TSI The type source information of the variable declaration.
     /// \returns The newly built variable declaration.
-    clang::VarDecl*
-    BuildVarDecl(clang::QualType Type, clang::IdentifierInfo* Identifier,
-                 clang::Expr* Init = nullptr, bool DirectInit = false,
-                 clang::TypeSourceInfo* TSI = nullptr,
-                 clang::VarDecl::InitializationStyle IS =
-                     clang::VarDecl::InitializationStyle::CInit);
+    clang::VarDecl* BuildVarDecl(clang::QualType Type,
+                                 clang::IdentifierInfo* Identifier,
+                                 clang::Expr* Init = nullptr,
+                                 bool DirectInit = false,
+                                 clang::TypeSourceInfo* TSI = nullptr);
     /// Builds variable declaration to be used inside the derivative
     /// body.
     /// \param[in] Type The type of variable declaration to build.
@@ -323,20 +348,18 @@ namespace clad {
     /// C style initalization.
     /// \param[in] TSI The type source information of the variable declaration.
     /// \returns The newly built variable declaration.
-    clang::VarDecl*
-    BuildVarDecl(clang::QualType Type, llvm::StringRef prefix = "_t",
-                 clang::Expr* Init = nullptr, bool DirectInit = false,
-                 clang::TypeSourceInfo* TSI = nullptr,
-                 clang::VarDecl::InitializationStyle IS =
-                     clang::VarDecl::InitializationStyle::CInit);
+    clang::VarDecl* BuildVarDecl(clang::QualType Type,
+                                 llvm::StringRef prefix = "_t",
+                                 clang::Expr* Init = nullptr,
+                                 bool DirectInit = false,
+                                 clang::TypeSourceInfo* TSI = nullptr);
     /// Builds variable declaration to be used inside the derivative
     /// body in the derivative function global scope.
-    clang::VarDecl*
-    BuildGlobalVarDecl(clang::QualType Type, llvm::StringRef prefix = "_t",
-                       clang::Expr* Init = nullptr, bool DirectInit = false,
-                       clang::TypeSourceInfo* TSI = nullptr,
-                       clang::VarDecl::InitializationStyle IS =
-                           clang::VarDecl::InitializationStyle::CInit);
+    clang::VarDecl* BuildGlobalVarDecl(clang::QualType Type,
+                                       llvm::StringRef prefix = "_t",
+                                       clang::Expr* Init = nullptr,
+                                       bool DirectInit = false,
+                                       clang::TypeSourceInfo* TSI = nullptr);
     /// Creates a namespace declaration and enters its context. All subsequent
     /// Stmts are built inside that namespace, until
     /// m_Sema.PopDeclContextIsUsed.
@@ -362,10 +385,12 @@ namespace clad {
     /// declaration reference expressions. This function builds a declaration
     /// reference given a declaration.
     /// \param[in] D The declaration to build a DeclRefExpr for.
-    /// \param[in] SS The scope specifier for the declaration.
+    /// \param[in] SS The nested name specifier for the declaration.
     /// \returns the DeclRefExpr for the given declaration.
-    clang::DeclRefExpr* BuildDeclRef(clang::DeclaratorDecl* D,
-                                     const clang::CXXScopeSpec* SS = nullptr);
+    clang::DeclRefExpr*
+    BuildDeclRef(clang::DeclaratorDecl* D,
+                 clang::NestedNameSpecifier* NNS = nullptr,
+                 clang::ExprValueKind VK = clang::VK_LValue);
 
     /// Stores the result of an expression in a temporary variable (of the same
     /// type as is the result of the expression) and returns a reference to it.
@@ -375,24 +400,14 @@ namespace clad {
     /// direct references in intermediate variables)
     clang::Expr* StoreAndRef(clang::Expr* E, Stmts& block,
                              llvm::StringRef prefix = "_t",
-                             bool forceDeclCreation = false,
-                             clang::VarDecl::InitializationStyle IS =
-                                 clang::VarDecl::InitializationStyle::CInit);
+                             bool forceDeclCreation = false);
     /// A shorthand to store directly to the current block.
     clang::Expr* StoreAndRef(clang::Expr* E, llvm::StringRef prefix = "_t",
-                             bool forceDeclCreation = false,
-                             clang::VarDecl::InitializationStyle IS =
-                                 clang::VarDecl::InitializationStyle::CInit);
+                             bool forceDeclCreation = false);
     /// An overload allowing to specify the type for the variable.
     clang::Expr* StoreAndRef(clang::Expr* E, clang::QualType Type, Stmts& block,
                              llvm::StringRef prefix = "_t",
-                             bool forceDeclCreation = false,
-                             clang::VarDecl::InitializationStyle IS =
-                                 clang::VarDecl::InitializationStyle::CInit);
-    /// For an expr E, decides if it is useful to store it in a temporary
-    /// variable and replace E's further usage by a reference to that variable
-    /// to avoid recomputation.
-    static bool UsefulToStore(clang::Expr* E);
+                             bool forceDeclCreation = false);
     /// A flag for silencing warnings/errors output by diag function.
     /// Shorthand to issues a warning or error.
     template <std::size_t N>
@@ -431,24 +446,6 @@ namespace clad {
       llvm::SmallVector<clang::Expr*, 1> IS = {Idx};
       return BuildArraySubscript(Base, IS);
     }
-    /// Find namespace clad declaration.
-    clang::NamespaceDecl* GetCladNamespace();
-    /// Find declaration of clad::class templated type
-    ///
-    /// \param[in] className name of the class to be found
-    /// \returns The declaration of the class with the name ClassName
-    clang::TemplateDecl*
-    LookupTemplateDeclInCladNamespace(llvm::StringRef ClassName);
-    /// Instantiate clad::class<TemplateArgs> type
-    ///
-    /// \param[in] CladClassDecl the decl of the class that is going to be used
-    /// in the creation of the type \param[in] TemplateArgs an array of template
-    /// arguments \returns The created type clad::class<TemplateArgs>
-    clang::QualType
-    InstantiateTemplate(clang::TemplateDecl* CladClassDecl,
-                        llvm::ArrayRef<clang::QualType> TemplateArgs);
-    clang::QualType InstantiateTemplate(clang::TemplateDecl* CladClassDecl,
-                                        clang::TemplateArgumentListInfo& TLI);
     /// Find declaration of clad::tape templated type.
     clang::TemplateDecl* GetCladTapeDecl();
     /// Perform a lookup into clad namespace for an entity with given name.
@@ -462,6 +459,21 @@ namespace clad {
     /// Instantiate clad::tape<T> type.
     clang::QualType GetCladTapeOfType(clang::QualType T);
 
+    /// Helper to build a function call expression.
+    ///
+    /// \param[in] funcName The name of the function to build the expression
+    /// for.
+    /// \param[in] nmspace The name of the namespace for the function,
+    /// currently does not support nested namespaces.
+    /// \param[in] callArgs A vector of \c clang::Expr of all the parameters
+    /// to the function call.
+    ///
+    /// \return The function call expression that can be used to emit into
+    /// code.
+    clang::Expr* GetFunctionCall(const std::string& funcName,
+                                 const std::string& nmspace,
+                                 llvm::SmallVectorImpl<clang::Expr*>& callArgs);
+
     clang::DeclRefExpr* GetCladTapePushDRE();
 
     clang::Stmt* GetCladZeroInit(llvm::MutableArrayRef<clang::Expr*> args);
@@ -473,12 +485,11 @@ namespace clad {
     void PerformImplicitConversionAndAssign(clang::VarDecl* VD,
                                             clang::Expr* Init) {
       // Implicitly convert Init into the type of VD
-      clang::ActionResult<clang::Expr*> ICAR = m_Sema
-          .PerformImplicitConversion(Init, VD->getType(),
-                                     clang::Sema::AA_Casting);
+      clang::ActionResult<clang::Expr*> ICAR = m_Sema.PerformImplicitConversion(
+          Init, VD->getType(), CLAD_COMPAT_CLANG20_SemaAACasting);
       assert(!ICAR.isInvalid() && "Invalid implicit conversion!");
       // Assign the resulting expression to the variable declaration
-      VD->setInit(ICAR.get());
+      SetDeclInit(VD, ICAR.get());
     }
 
     /// Build a call to member function through Base expr and using the function
@@ -497,7 +508,15 @@ namespace clad {
     BuildCallExprToMemFn(clang::Expr* Base, llvm::StringRef MemberFunctionName,
                          llvm::MutableArrayRef<clang::Expr*> ArgExprs,
                          clang::SourceLocation Loc = noLoc);
+    clang::Expr*
+    BuildCallExprToMemFn(clang::Expr* Base,
+                         clang::UnqualifiedId* MemberFunction,
+                         llvm::MutableArrayRef<clang::Expr*> ArgExprs,
+                         clang::SourceLocation Loc = noLoc);
 
+    // FIXME: This overload is only used because it builds calls to methods
+    // that are not yet added to the class, and therefore, we cannot perform
+    // a lookup.
     /// Build a call to member function through this pointer.
     ///
     /// \param[in] FD callee member function
@@ -519,10 +538,10 @@ namespace clad {
     /// perfectly forwarded while calling member functions.
     /// \returns Built call expression
     clang::Expr*
-    BuildCallExprToFunction(clang::FunctionDecl* FD,
+    BuildCallExprToFunction(const clang::FunctionDecl* FD,
                             llvm::MutableArrayRef<clang::Expr*> argExprs,
-                            bool useRefQualifiedThisObj = false,
-                            const clang::CXXScopeSpec* SS = nullptr);
+                            clang::Expr* CUDAExecConfig = nullptr,
+                            bool useRefQualifiedThisObj = false);
 
     /// Build a call to templated free function inside the clad namespace.
     ///
@@ -536,20 +555,8 @@ namespace clad {
         llvm::ArrayRef<clang::TemplateArgument> templateArgs,
         clang::SourceLocation loc);
 
-    /// Find declaration of clad::array_ref templated type.
-    clang::TemplateDecl* GetCladArrayRefDecl();
-    /// Create clad::array_ref<T> type.
-    clang::QualType GetCladArrayRefOfType(clang::QualType T);
     /// Checks if the type is of clad::array<T> or clad::array_ref<T> type
     bool isCladArrayType(clang::QualType QT);
-    /// Find declaration of clad::array templated type.
-    clang::TemplateDecl* GetCladArrayDecl();
-    /// Create clad::array<T> type.
-    clang::QualType GetCladArrayOfType(clang::QualType T);
-    /// Find declaration of clad::matrix templated type.
-    clang::TemplateDecl* GetCladMatrixDecl();
-    /// Create clad::matrix<T> type.
-    clang::QualType GetCladMatrixOfType(clang::QualType T);
     /// Creates the expression clad::matrix<T>::identity(Args) for the given
     /// type and args.
     clang::Expr*
@@ -581,7 +588,8 @@ namespace clad {
     /// \returns The derivative function call.
     clang::Expr* GetSingleArgCentralDiffCall(
         clang::Expr* targetFuncCall, clang::Expr* targetArg, unsigned targetPos,
-        unsigned numArgs, llvm::SmallVectorImpl<clang::Expr*>& args);
+        unsigned numArgs, llvm::SmallVectorImpl<clang::Expr*>& args,
+        clang::Expr* CUDAExecConfig = nullptr);
 
     /// Emits diagnostic messages on differentiation (or lack thereof) for
     /// call expressions.
@@ -592,14 +600,20 @@ namespace clad {
                                  clang::SourceLocation srcLoc);
 
     clang::QualType DetermineCladArrayValueType(clang::QualType T);
-
-    /// Returns clad::Identify template declaration.
-    clang::TemplateDecl* GetCladConstructorPushforwardTag();
-
-    /// Returns type clad::Identify<T>
-    clang::QualType GetCladConstructorPushforwardTagOfType(clang::QualType T);
+    /// Find the derived function if present in the DerivedFnCollector.
+    ///
+    /// \param[in] request The request to find the derived function.
+    ///
+    /// \returns The derived function if found, nullptr otherwise.
+    clang::FunctionDecl* FindDerivedFunction(DiffRequest& request);
 
   public:
+    /// Builds an overload for the derivative function that has derived params
+    /// for all the arguments of the requested function and it calls the
+    /// original derivative function internally. Used in gradient and jacobian
+    /// modes.
+    clang::FunctionDecl*
+    CreateDerivativeOverload(clang::FunctionDecl derivative);
     /// Rebuild a sequence of nested namespaces ending with DC.
     clang::NamespaceDecl* RebuildEnclosingNamespaces(clang::DeclContext* DC);
     /// Clones a statement
@@ -610,6 +624,24 @@ namespace clad {
     /// store a pointer to their size expression.
     clang::QualType CloneType(clang::QualType T);
 
+    /// Initiates the differentiation process.
+    /// Returns the derivative and its overload, if any.
+    virtual DerivativeAndOverload Derive() = 0;
+
+    /// Builds the QualType of the derivative to be generated.
+    ///
+    /// \param[in] forCustomDerv If true, turns member functions into regular
+    /// functions by moving the base to the parameters.
+    clang::QualType
+    GetDerivativeType(llvm::ArrayRef<clang::QualType> customParams = {});
+
+    /// Builds an overload for the derivative function that has derived params
+    /// for all the arguments of the requested function and it calls the
+    /// original derivative function internally. Used in gradient and jacobian
+    /// modes.
+    clang::FunctionDecl*
+    CreateDerivativeOverload(clang::FunctionDecl* derivative = nullptr);
+
     /// Computes effective derivative operands. It should be used when operands
     /// might be of pointer types.
     ///
@@ -644,9 +676,25 @@ namespace clad {
                                    clang::Expr*& derivedL,
                                    clang::Expr*& derivedR);
 
-  private:
-    clang::TemplateDecl* m_CladConstructorPushforwardTag = nullptr;
+    virtual ~VisitorBase() = 0;
   };
+
+  /// A class that generates prettier stack traces when we crash on generating
+  /// a derivative.
+  class PrettyStackTraceDerivative : public llvm::PrettyStackTraceEntry {
+    const DiffRequest& m_DiffReq;
+    using Blocks = std::vector<llvm::SmallVector<clang::Stmt*, 16>>;
+    const Blocks& m_Blocks;
+    const clang::Sema& m_Sema;
+    const clang::Stmt** m_Stmt = nullptr;
+
+  public:
+    PrettyStackTraceDerivative(const DiffRequest& DiffReq, const Blocks& B,
+                               const clang::Sema& Sema, const clang::Stmt** S)
+        : m_DiffReq(DiffReq), m_Blocks(B), m_Sema(Sema), m_Stmt(S) {}
+    void print(llvm::raw_ostream& OS) const override;
+  };
+
 } // end namespace clad
 
 #endif // CLAD_VISITOR_BASE_H
