@@ -8,8 +8,6 @@
  *                                                                    *
  **********************************************************************/
 
-
-
 /**
 
 Probability density functions, cumulative distribution functions
@@ -23,15 +21,11 @@ user must calculate the shift themselves if they wish.
 
 MathCore provides the majority of the probability density functions, of the
 cumulative distributions and of the quantiles (inverses of the cumulatives).
-Additional distributions are also provided by the
-<A HREF="../../MathMore/html/group__StatFunc.html">MathMore</A> library.
+Additional distributions are also provided by the \ref MathMore library.
 
 
 @defgroup StatFunc Statistical functions
-
 @ingroup  MathCore
-@ingroup  MathMore
-
 */
 
 #ifndef ROOT_Math_PdfFuncMathCore
@@ -58,12 +52,11 @@ namespace Math {
    *
    */
 
-   /** @name Probability Density Functions from MathCore
-   *   Additional PDF's are provided in the MathMore library
-   *   (see PDF functions from MathMore)
-   */
+/** @name Probability Density Functions from MathCore
+ *   Additional PDFs are provided in the MathMore library when the GSL is available.
+ */
 
-  //@{
+//@{
 
   /**
 
@@ -409,7 +402,7 @@ namespace Math {
 
   Probability density function of the bi-dimensional (Gaussian) distribution.
 
-  \f[ p(x) = {1 \over 2 \pi \sigma_x \sigma_y \sqrt{1-\rho^2}} \exp (-((x-x0)^2/\sigma_x^2 + (y-y0)^2/\sigma_y^2 - 2 \rho x y/(\sigma_x\sigma_y))/2(1-\rho^2)) \f]
+  \f[ p(x,y) = {1 \over 2 \pi \sigma_x \sigma_y \sqrt{1-\rho^2}} \exp (-((x-x0)^2/\sigma_x^2 + (y-y0)^2/\sigma_y^2 - 2 \rho x y/(\sigma_x\sigma_y))/2(1-\rho^2)) \f]
 
   For detailed description see
   <A HREF="http://mathworld.wolfram.com/BivariateNormalDistribution.html">
