@@ -36,7 +36,7 @@ protected:
    THn() = default;
    THn(const char* name, const char* title, Int_t dim, const Int_t* nbins,
        const Double_t* xmin, const Double_t* xmax);
-
+   THn(const char *name, const char *title, const std::vector<TAxis> &axes);
    THn(const char *name, const char *title, Int_t dim, const Int_t *nbins,
        const std::vector<std::vector<double>> &xbins);
 
@@ -226,6 +226,15 @@ public:
    THn(name, title, dim, nbins, xmin, xmax),
    fArray(dim, nbins, true)  {}
 
+   THnT(const char *name, const char *title, const std::vector<TAxis> &axes) : THn(name, title, axes)
+   {
+      const Int_t dim = axes.size();
+      std::vector<Int_t> nbins(dim);
+      for (Int_t i = 0; i < dim; i++)
+         nbins[i] = axes.at(i).GetNbins();
+      fArray = TNDArrayT<T>(dim, nbins.data(), true);
+   }
+
    THnT(const char *name, const char *title, Int_t dim, const Int_t *nbins,
         const std::vector<std::vector<double>> &xbins)
       : THn(name, title, dim, nbins, xbins), fArray(dim, nbins, true)
