@@ -9,12 +9,7 @@
 ################################################################################
 
 from . import pythonization
-from ROOT._pythonization._memory_utils import _constructor_releasing_ownership
-
-
-def inject_constructor_releasing_ownership(klass):
-    klass._cpp_constructor = klass.__init__
-    klass.__init__ = _constructor_releasing_ownership
+from ROOT._pythonization._memory_utils import inject_constructor_releasing_ownership
 
 
 # The constructors need to be pythonized for each derived class separately:
@@ -34,3 +29,8 @@ _th2_derived_classes_to_pythonize = [
 
 for klass in _th2_derived_classes_to_pythonize:
     pythonization(klass)(inject_constructor_releasing_ownership)
+
+    from ROOT._pythonization._uhi import _add_plotting_features
+
+    # Add UHI plotting features
+    pythonization(klass)(_add_plotting_features)
